@@ -58,20 +58,27 @@ static int str_hostname_cb(void *data, const char *input);
 static struct fio_option options[] = {
 	{
 		.name	= "hostname",
+		.lname	= "net engine hostname",
 		.type	= FIO_OPT_STR_STORE,
 		.cb	= str_hostname_cb,
 		.help	= "Hostname for net IO engine",
+		.category = FIO_OPT_C_ENGINE,
+		.group	= FIO_OPT_G_NETIO,
 	},
 	{
 		.name	= "port",
+		.lname	= "net engine port",
 		.type	= FIO_OPT_INT,
 		.off1	= offsetof(struct netio_options, port),
 		.minval	= 1,
 		.maxval	= 65535,
 		.help	= "Port to use for TCP or UDP net connections",
+		.category = FIO_OPT_C_ENGINE,
+		.group	= FIO_OPT_G_NETIO,
 	},
 	{
 		.name	= "protocol",
+		.lname	= "net engine protocol",
 		.alias	= "proto",
 		.type	= FIO_OPT_STR,
 		.off1	= offsetof(struct netio_options, proto),
@@ -91,6 +98,8 @@ static struct fio_option options[] = {
 			    .help = "UNIX domain socket",
 			  },
 		},
+		.category = FIO_OPT_C_ENGINE,
+		.group	= FIO_OPT_G_NETIO,
 	},
 #ifdef CONFIG_TCP_NODELAY
 	{
@@ -98,19 +107,26 @@ static struct fio_option options[] = {
 		.type	= FIO_OPT_BOOL,
 		.off1	= offsetof(struct netio_options, nodelay),
 		.help	= "Use TCP_NODELAY on TCP connections",
+		.category = FIO_OPT_C_ENGINE,
+		.group	= FIO_OPT_G_NETIO,
 	},
 #endif
 	{
 		.name	= "listen",
+		.lname	= "net engine listen",
 		.type	= FIO_OPT_STR_SET,
 		.off1	= offsetof(struct netio_options, listen),
 		.help	= "Listen for incoming TCP connections",
+		.category = FIO_OPT_C_ENGINE,
+		.group	= FIO_OPT_G_NETIO,
 	},
 	{
 		.name	= "pingpong",
 		.type	= FIO_OPT_STR_SET,
 		.off1	= offsetof(struct netio_options, pingpong),
 		.help	= "Ping-pong IO requests",
+		.category = FIO_OPT_C_ENGINE,
+		.group	= FIO_OPT_G_NETIO,
 	},
 	{
 		.name	= NULL,
@@ -970,7 +986,7 @@ static struct ioengine_ops ioengine_rw = {
 	.options		= options,
 	.option_struct_size	= sizeof(struct netio_options),
 	.flags			= FIO_SYNCIO | FIO_DISKLESSIO | FIO_UNIDIR |
-				  FIO_PIPEIO,
+				  FIO_PIPEIO | FIO_BIT_BASED,
 };
 
 static int str_hostname_cb(void *data, const char *input)
