@@ -1302,6 +1302,11 @@ struct fio_option fio_options[FIO_MAX_OPTS] = {
 			  { .ival = "vsync",
 			    .help = "Use readv/writev",
 			  },
+#ifdef CONFIG_PWRITEV
+			  { .ival = "pvsync",
+			    .help = "Use preadv/pwritev",
+			  },
+#endif
 #ifdef CONFIG_LIBAIO
 			  { .ival = "libaio",
 			    .help = "Linux native asynchronous IO",
@@ -2773,7 +2778,7 @@ struct fio_option fio_options[FIO_MAX_OPTS] = {
 	{
 		.name	= "write_iops_log",
 		.lname	= "Write IOPS log",
-		.type	= FIO_OPT_STR,
+		.type	= FIO_OPT_STR_STORE,
 		.off1	= td_var_offset(iops_log_file),
 		.help	= "Write log of IOPS during run",
 		.category = FIO_OPT_C_LOG,
@@ -2819,7 +2824,7 @@ struct fio_option fio_options[FIO_MAX_OPTS] = {
 	{
 		.name	= "group_reporting",
 		.lname	= "Group reporting",
-		.type	= FIO_OPT_BOOL,
+		.type	= FIO_OPT_STR_SET,
 		.off1	= td_var_offset(group_reporting),
 		.help	= "Do reporting on a per-group basis",
 		.category = FIO_OPT_C_STAT,
